const chai = require('chai');
const chaiHttp = require('chai-http');
const fs = require('fs');
const path = require('path');
const app = require('../index');
const expect = chai.expect;

chai.use(chaiHttp);

let authToken;
let adId;

describe('Ad Routes', () => {
    before(async () => {
        try {
            const loginResponse = await chai
                .request(app)
                .post('/user/login')
                .send({ username: 'hajar', password: 'hajar' });

            authToken = loginResponse.body.token;
        } catch (error) {
            console.error('Error obtaining authentication token:', error);
        }
    });

    describe('POST /ads', () => {
        it('should add a new ad', async () => {
            const response = await chai
                .request(app)
                .post('/ad')
                .set('Authorization', `Bearer ${authToken}`)
                .send({
                    title: 'Test Ad',
                    propertyType: 'À la vente',
                    publicationStatus: 'Publiée',
                    propertyStatus: 'Disponible',
                    description: 'Test description',
                    price: 80000,
                    availabilityDate: '2024-12-01',
                });
            expect(response.body).to.have.property('_id').to.be.a('string');
            adId = response.body._id;
        });

    const backupFolderPath = path.join(__dirname, '..', 'backup');

        it('should add a new ad with photos', async () => {
            const response = await chai
                .request(app)
                .post('/ad')
                .set('Authorization', `Bearer ${authToken}`)
                .attach('photos', path.join(backupFolderPath, 'img1.jpg'))
                .attach('photos', path.join(backupFolderPath, 'img2.jpg'))
                .field('title', 'Test Ad 2')
                .field('propertyType', 'À la vente')
                .field('publicationStatus', 'Publiée')
                .field('propertyStatus', 'Disponible')
                .field('description', 'teest.')
                .field('price', 80000)
                .field('availabilityDate', '2024-11-01');

            expect(response).to.have.status(200);
            expect(response.body).to.have.property('_id').to.be.a('string');
            adId = response.body._id;
        });

        it('should not allow a non-authorized user (non-agent) to add a new ad', async () => {
            // Log in as a non-agent user
            const nonAgentLoginResponse = await chai
                .request(app)
                .post('/user/login')
                .send({ username: 'manal', password: 'manal' });

            const nonAgentAuthToken = nonAgentLoginResponse.body.token;

            // Attempt to add a new ad as a non-agent user
            const response = await chai
                .request(app)
                .post('/ad')
                .set('Authorization', `Bearer ${nonAgentAuthToken}`)
                .send({
                    title: 'Test Ad',
                    propertyType: 'À la vente',
                    publicationStatus: 'Publiée',
                    propertyStatus: 'Disponible',
                    description: 'Test description',
                    price: 80000,
                    availabilityDate: '2024-12-01',
                });

            expect(response).to.have.status(403);
            expect(response.body).to.have.property('message').to.equal('User non autorisé');

            const logoutResponse = await chai
                .request(app)
                .post('/user/logout')
                .set('Authorization', `Bearer ${nonAgentAuthToken}`);

            expect(logoutResponse).to.have.status(200);
        });

    });

    describe('PUT /ads/:id', () => {
        it('should update an existing ad with the agent user (hajar)', async () => {
            // Log in as the agent user (hajar)
            const agentLoginResponse = await chai
                .request(app)
                .post('/user/login')
                .send({ username: 'hajar', password: 'hajar' });

            const agentAuthToken = agentLoginResponse.body.token;

            // Create a new ad to update
            const createResponse = await chai
                .request(app)
                .post('/ad')
                .set('Authorization', `Bearer ${agentAuthToken}`)
                .send({
                    title: 'Test Ad to update',
                    propertyType: 'À la vente',
                    publicationStatus: 'Publiée',
                    propertyStatus: 'Disponible',
                    description: 'Test description',
                    price: 80000,
                    availabilityDate: '2024-12-01',
                });

            const adIdToUpdate = createResponse.body._id;

            // Update the ad with the agent user (hajar)
            const updateResponse = await chai
                .request(app)
                .put(`/ad/${adIdToUpdate}`)
                .set('Authorization', `Bearer ${agentAuthToken}`)
                .send({
                    title: 'Updated Test Ad',
                    description: 'Updated description',
                    price: 90000,
                });
            expect(updateResponse).to.have.status(200);
            expect(updateResponse.body).to.have.property('_id').to.equal(adIdToUpdate);
            expect(updateResponse.body).to.have.property('title').to.equal('Updated Test Ad');
            expect(updateResponse.body).to.have.property('description').to.equal('Updated description');
            expect(updateResponse.body).to.have.property('price').to.equal(90000);

            // Log out the agent user (hajar)
            const logoutResponse = await chai
                .request(app)
                .post('/user/logout')
                .set('Authorization', `Bearer ${agentAuthToken}`);

            expect(logoutResponse).to.have.status(200);
        });
    });


    describe('GET /ads', () => {
        it('should get all ads', async () => {
            const response = await chai
                .request(app)
                .get('/ad')
                .set('Authorization', `Bearer ${authToken}`);

            expect(response).to.have.status(200);
            expect(response.body).to.be.an('array');
        });
    });

    // Cleanup after each test
    afterEach(async () => {
        try {
            // Delete the ad if adId is defined
            if (adId) {
                await chai
                    .request(app)
                    .delete(`/ad/${adId}`)
                    .set('Authorization', `Bearer ${authToken}`);
            }

            const photo1Path = path.join(__dirname, 'uploads', 'img1.jpg');
            const photo2Path = path.join(__dirname, 'uploads', 'img2.jpg');

            // Delete the photos
            if (fs.existsSync(photo1Path)) {
                fs.unlinkSync(photo1Path);
            }

            if (fs.existsSync(photo2Path)) {
                fs.unlinkSync(photo2Path);
            }
        } catch (error) {
            console.error('Error cleaning up:', error);
        }
    });
});
